@@ -7,8 +7,10 @@ use strict;
 
 package XML::Compile::Schema::Translate;
 use vars '$VERSION';
-$VERSION = '0.65';
+$VERSION = '0.66';
 
+# Errors are either in class 'usage': called with request
+#                         or 'schema': syntax error in schema
 use Log::Report 'xml-compile', syntax => 'SHORT';
 use List::Util  'first';
 
@@ -58,7 +60,7 @@ sub compileTree($@)
            $name eq 'element'
         or $name eq 'attribute'
         or error __x"ID {id} must be an element or attribute, but is {name}"
-              , id => $element, name => $name;
+              , id => $element, name => $name, class => 'usage';
 
         $element  = $def->{full};
     }
@@ -78,7 +80,8 @@ sub assertType($$$$)
     return if $checker->($value);
 
     error __x"field {field} contains '{value}' which is not a valid {type} at {where}"
-        , field => $field, value => $value, type => $type, where => $where;
+        , field => $field, value => $value, type => $type, where => $where
+        , class => 'usage';
 
 }
 
@@ -123,7 +126,7 @@ sub topLevel($$)
        or error __x(( $fullname eq $path
                     ? N__"cannot find element or attribute `{name}'"
                     : N__"cannot find element or attribute `{name}' at {where}"
-                    ), name => $fullname, where => $path);
+                    ), name => $fullname, where => $path, class => 'usage');
 
     my $node = $top->{node};
 
@@ -168,7 +171,8 @@ sub topLevel($$)
       = $name eq 'element'   ? $self->element($tree)
       : $name eq 'attribute' ? $self->attributeOne($tree)
       : error __x"top-level {full} is not an element or attribute but {name} at {where}"
-            , full => $fullname, name => $name, where => $tree->path;
+            , full => $fullname, name => $name, where => $tree->path
+            , class => 'usage';
 
     my $wrapper = $name eq 'element' ? 'element_wrapper' : 'attribute_wrapper';
     $self->make($wrapper, $path, $make);
@@ -200,7 +204,7 @@ sub typeByName($$)
     my $top    = $self->namespaces->find(complexType => $typename)
               || $self->namespaces->find(simpleType  => $typename)
        or error __x"cannot find type {type} at {where}"
-              , type => $typename, where => $tree->path;
+            , type => $typename, where => $tree->path, class => 'usage';
 
     my $elems_qual
      = exists $self->{elements_qualified} ? $self->{elements_qualified}
@@ -221,7 +225,7 @@ sub typeByName($$)
       $typedef eq 'simpleType'  ? $self->simpleType($typeimpl)
     : $typedef eq 'complexType' ? $self->complexType($typeimpl)
     : error __x"expecting simple- or complexType, not '{type}' at {where}"
-          , type => $typedef, where => $tree->path;
+          , type => $typedef, where => $tree->path, class => 'schema';
 }
 
 sub simpleType($;$)
@@ -229,7 +233,7 @@ sub simpleType($;$)
 
     $tree->nrChildren==1
        or error __x"simpleType must have exactly one child at {where}"
-              , where => $tree->path;
+            , where => $tree->path, class => 'schema';
 
     my $child = $tree->firstChild;
     my $name  = $child->localName;
@@ -244,7 +248,7 @@ sub simpleType($;$)
     : $name eq 'list'        ? $self->simpleList($nest)
     : $name eq 'union'       ? $self->simpleUnion($nest)
     : error __x"simpleType contains '{local}', must be restriction, list, or union at {where}"
-          , local => $name, where => $tree->path;
+          , local => $name, where => $tree->path, class => 'schema';
 
     delete @$type{'attrs','attrs_any'};  # spec says ignore attrs
     $type;
@@ -263,7 +267,7 @@ sub simpleList($)
     if(my $type = $node->getAttribute('itemType'))
     {   $tree->nrChildren==0
             or error __x"list with both itemType and content at {where}"
-                   , where => $where;
+                 , where => $where, class => 'schema';
 
         $self->assertType($where, itemType => QName => $type);
         my $typename = $self->rel2abs($where, $node, $type);
@@ -272,11 +276,11 @@ sub simpleList($)
     else
     {   $tree->nrChildren==1
             or error __x"list expects one simpleType child at {where}"
-                   , where => $where;
+                 , where => $where, class => 'schema';
 
         $tree->currentLocal eq 'simpleType'
             or error __x"list can only have a simpleType child at {where}"
-                   , where => $where;
+                 , where => $where, class => 'schema';
 
         $per_item    = $self->simpleType($tree->descend, 1);
     }
@@ -313,7 +317,7 @@ sub simpleUnion($)
             my $type = $self->typeByName($tree, $typename);
             my $st   = $type->{st}
                 or error __x"union only of simpleTypes, but {type} is complex at {where}"
-                       , type => $typename, where => $where;
+                     , type => $typename, where => $where, class => 'schema';
 
             push @types, $st;
         }
@@ -323,7 +327,7 @@ sub simpleUnion($)
     {   my $name = $child->localName;
         $name eq 'simpleType'
             or error __x"only simpleType's within union, found {local} at {where}"
-                   , local => $name, where => $where;
+                 , local => $name, where => $where, class => 'schema';
 
         my $ctype = $self->simpleType($tree->descend($child), 0);
         push @types, $ctype->{st};
@@ -351,11 +355,11 @@ sub simpleRestriction($$)
     else
     {   my $simple   = $tree->firstChild
             or error __x"no base in simple-restriction, so simpleType required at {where}"
-                   , where => $where;
+                   , where => $where, class => 'schema';
 
         $simple->localName eq 'simpleType'
             or error __x"simpleType expected, because there is no base attribute at {where}"
-                   , where => $where;
+                   , where => $where, class => 'schema';
 
         $base = $self->simpleType($tree->descend($simple, 'st'));
         $tree->nextChild;
@@ -363,13 +367,13 @@ sub simpleRestriction($$)
 
     my $st = $base->{st}
         or error __x"simple-restriction is not a simpleType at {where}"
-               , where => $where;
+               , where => $where, class => 'schema';
 
     my $do = $self->applySimpleFacets($tree, $st, $in_list);
 
     $tree->currentChild
         and error __x"elements left at tail at {where}"
-                , where => $tree->path;
+                , where => $tree->path, class => 'schema';
 
     +{ st => $do };
 }
@@ -392,14 +396,14 @@ sub applySimpleFacets($$$)
         my $value = $child->getAttribute('value');
         defined $value
             or error __x"no value for facet `{facet}' at {where}"
-                   , facet => $facet, where => $where;
+                   , facet => $facet, where => $where, class => 'schema';
 
            if($facet eq 'enumeration') { push @{$facets{enumeration}}, $value }
         elsif($facet eq 'pattern')     { push @{$facets{pattern}}, $value }
         elsif(!exists $facets{$facet}) { $facets{$facet} = $value }
         else
         {   error __x"facet `{facet}' defined twice at {where}"
-                , facet => $facet, where => $where;
+                , facet => $facet, where => $where, class => 'schema';
         }
     }
 
@@ -446,7 +450,8 @@ sub element($)
 
     my $node     = $tree->node;
     my $name     = $node->getAttribute('name')
-        or error __x"element has no name at {where}", where => $tree->path;
+        or error __x"element has no name at {where}"
+             , where => $tree->path, class => 'schema';
 
     $self->assertType($tree->path, name => NCName => $name);
     my $fullname = pack_type $self->{tns}, $name;
@@ -467,7 +472,7 @@ sub element($)
       : $form eq 'qualified'   ? 1
       : $form eq 'unqualified' ? 0
       : error __x"form must be (un)qualified, not `{form}' at {where}"
-            , form => $form, where => $tree->path;
+            , form => $form, where => $tree->path, class => 'schema';
 
     my $trans     = $qual ? 'tag_qualified' : 'tag_unqualified';
     my $tag       = $self->make($trans => $where, $node, $name);
@@ -477,7 +482,7 @@ sub element($)
     if(my $isa = $node->getAttribute('type'))
     {   $nr_childs==0
             or error __x"no childs expected with attribute `type' at {where}"
-                   , where => $where;
+                   , where => $where, class => 'schema';
 
         $self->assertType($where, type => QName => $isa);
         $typename = $self->rel2abs($where, $node, $isa);
@@ -488,7 +493,8 @@ sub element($)
         $type     = $self->typeByName($tree, $typename);
     }
     elsif($nr_childs!=1)
-    {   error __x"expected is only one child at {where}", where => $where;
+    {   error __x"expected is only one child at {where}"
+          , where => $where, class => 'schema';
     }
     else # nameless types
     {   my $child = $tree->firstChild;
@@ -499,7 +505,7 @@ sub element($)
           = $local eq 'simpleType'  ? $self->simpleType($nest, 0)
           : $local eq 'complexType' ? $self->complexType($nest)
           : error __x"unexpected element child `{name}' and {where}"
-                , name => $local, where => $where;
+                , name => $local, where => $where, class => 'schema';
     }
 
     my ($before, $replace, $after)
@@ -554,6 +560,9 @@ sub particle($)
     $max = 'unbounded'
         if $max ne 'unbounded' && $max > 1 && !$self->{check_occurs};
 
+    return ()
+        if $max ne 'unbounded' && $max==0;
+
     $min = 0
         if $max eq 'unbounded' && !$self->{check_occurs};
 
@@ -565,7 +574,7 @@ sub particle($)
       : $local eq 'group'          ? $self->particleGroup($tree)
       : $local =~ $particle_blocks ? $self->particleBlock($tree)
       : error __x"unknown particle type '{name}' at {where}"
-            , name => $local, where => $tree->path;
+            , name => $local, where => $tree->path, class => 'schema';
 
     defined $label
         or return ();
@@ -591,26 +600,27 @@ sub particleGroup($)
     my $node  = $tree->node;
     my $where = $tree->path . '#group';
     my $ref   = $node->getAttribute('ref')
-        or error __x"group without ref at {where}", where => $where;
+        or error __x"group without ref at {where}"
+             , where => $where, class => 'schema';
 
     $self->assertType($tree, ref => QName => $ref);
     my $typename = $self->rel2abs($where, $node, $ref);
 
     my $dest    = $self->namespaces->find(group => $typename)
         or error __x"cannot find group `{name}' at {where}"
-               , name => $typename, where => $where;
+             , name => $typename, where => $where, class => 'schema';
 
     my $group   = $tree->descend($dest->{node});
     return {} if $group->nrChildren==0;
 
     $group->nrChildren==1
         or error __x"only one particle block expected in group `{name}' at {where}"
-               , name => $typename, where => $where;
+               , name => $typename, where => $where, class => 'schema';
 
     my $local = $group->currentLocal;
     $local    =~ m/^(?:all|choice|sequence)$/
         or error __x"illegal group member `{name}' at {where}"
-               , name => $local, where => $where;
+               , name => $local, where => $where, class => 'schema';
 
     $self->particleBlock($group->descend);
 }
@@ -637,7 +647,7 @@ sub particleElementSubst($)
 
     my $groupname = $node->getAttribute('name')
         or error __x"substitutionGroup element needs name at {where}"
-               , where => $tree->path;
+               , where => $tree->path, class => 'schema';
 
     $self->assertType($where, name => QName => $groupname);
  
@@ -649,7 +659,7 @@ sub particleElementSubst($)
     # at least the base is expected
     @subgrps
         or error __x"no substitutionGroups found for {type} at {where}"
-               , type => $type, where => $where;
+               , type => $type, where => $where, class => 'schema';
 
     my @elems = map { $self->particleElement($tree->descend($_)) } @subgrps;
 
@@ -667,7 +677,7 @@ sub particleElement($)
 
         my $def     = $self->namespaces->find(element => $refname)
             or error __x"cannot find element '{name}' at {where}"
-                   , name => $refname, where => $where;
+                   , name => $refname, where => $where, class => 'schema';
 
         my $refnode  = $def->{node};
         my $abstract = $refnode->getAttribute('abstract') || 'false';
@@ -680,7 +690,7 @@ sub particleElement($)
 
     my $name     = $node->getAttribute('name')
         or error __x"element needs name or ref at {where}"
-               , where => $tree->path;
+               , where => $tree->path, class => 'schema';
 
     my $where    = $tree->path . "/el($name)";
     my $default  = $node->getAttributeNode('default');
@@ -727,12 +737,12 @@ sub attributeOne($)
         my $refname = $self->rel2abs($tree, $node, $refattr);
         my $def     = $self->namespaces->find(attribute => $refname)
             or error __x"cannot find attribute {name} at {where}"
-                   , name => $refname, where => $tree->path;
+                 , name => $refname, where => $tree->path, class => 'schema';
 
         $ref        = $def->{node};
         $name       = $ref->getAttribute('name')
             or error __x"ref attribute without name at {where}"
-                   , where => $tree->path;
+                 , where => $tree->path, class => 'schema';
 
         if($typeattr = $ref->getAttribute('type'))
         {   # postpone interpretation
@@ -741,7 +751,7 @@ sub attributeOne($)
         {   my $other = $tree->descend($ref);
             $other->nrChildren==1 && $other->currentLocal eq 'simpleType'
                 or error __x"toplevel attribute {type} has no type attribute nor single simpleType child"
-                      , type => $refname;
+                     , type => $refname, class => 'schema';
             $type   = $self->simpleType($other->descend);
         }
         $form       = $ref->getAttribute('form');
@@ -749,7 +759,8 @@ sub attributeOne($)
     elsif($tree->nrChildren==1)
     {   $tree->currentLocal eq 'simpleType'
             or error __x"attribute child can only be `simpleType', not `{found}' at {where}"
-                  , found => $tree->currentLocal, where => $tree->path;
+                 , found => $tree->currentLocal, where => $tree->path
+                 , class => 'schema';
 
         $name       = $node->getAttribute('name')
             or error __x"attribute without name at {where}"
@@ -762,7 +773,7 @@ sub attributeOne($)
     else
     {   $name       = $node->getAttribute('name')
             or error __x"attribute without name or ref at {where}"
-                   , where => $tree->path;
+                   , where => $tree->path, class => 'schema';
 
         $typeattr   = $node->getAttribute('type');
         $form       = $node->getAttribute('form');
@@ -785,14 +796,14 @@ sub attributeOne($)
 
     my $st      = $type->{st}
         or error __x"attribute not based in simple value type at {where}"
-               , where => $where;
+             , where => $where, class => 'schema';
 
     my $qual
       = ! defined $form        ? $self->{attrs_qual}
       : $form eq 'qualified'   ? 1
       : $form eq 'unqualified' ? 0
       : error __x"form must be (un)qualified, not {form} at {where}"
-            , form => $form, where => $where;
+            , form => $form, where => $where, class => 'schema';
 
     my $trans   = $qual ? 'tag_qualified' : 'tag_unqualified';
     my $tag     = $self->make($trans => $path, $node, $name);
@@ -801,7 +812,7 @@ sub attributeOne($)
     my $use     = $node->getAttribute('use') || '';
     $use =~ m/^(?:optional|required|prohibited|)$/
         or error __x"attribute use is required, optional or prohibited (not '{use}') at {where}"
-               , use => $use, where => $where;
+             , use => $use, where => $where, class => 'schema';
 
     my $default = $node->getAttributeNode('default');
     my $fixed   = $node->getAttributeNode('fixed');
@@ -829,7 +840,7 @@ sub attributeGroup($)
     my $where = $tree->path;
     my $ref   = $node->getAttribute('ref')
         or error __x"attributeGroup use without ref at {where}"
-               , where => $tree->path;
+             , where => $tree->path, class => 'schema';
 
     $self->assertType($where, ref => QName => $ref);
 
@@ -837,7 +848,7 @@ sub attributeGroup($)
 
     my $def  = $self->namespaces->find(attributeGroup => $typename)
         or error __x"cannot find attributeGroup {name} at {where}"
-               , name => $typename, where => $where;
+             , name => $typename, where => $where, class => 'schema';
 
     $self->attributeList($tree->descend($def->{node}));
 }
@@ -949,7 +960,7 @@ sub complexType($)
 
     $tree->nrChildren==1
         or error __x"expected is single simpleContent or complexContent at {where}"
-               , where => $tree->path;
+             , where => $tree->path, class => 'schema';
 
     my $nest  = $tree->descend($first);
     return $self->simpleContent($nest)
@@ -958,8 +969,8 @@ sub complexType($)
     return  $self->complexContent($nest)
         if $name eq 'complexContent';
 
-    error __x"complexType contains particles, simpleContent, or complexContent, not '{name}' at {where}"
-        , name => $name, where => $tree->path;
+    error __x"complexType contains particles, simpleContent or complexContent, not `{name}' at {where}"
+      , name => $name, where => $tree->path, class => 'schema';
 }
 
 sub complexBody($)
@@ -982,7 +993,9 @@ sub complexBody($)
     my @attrs = $self->attributeList($tree);
 
     defined $tree->currentChild
-        and error __x"trailing non-attribute at {where}", where => $tree->path;
+        and error __x"trailing non-attribute `{name}' at {where}"
+              , name => $tree->currentChild->localName, where => $tree->path
+              , class => 'schema';
 
     {elems => \@elems, @attrs};
 }
@@ -1026,7 +1039,7 @@ sub simpleContent($)
 
     $tree->nrChildren==1
         or error __x"need one simpleContent child at {where}"
-             , where => $tree->path;
+             , where => $tree->path, class => 'schema';
 
     my $name  = $tree->currentLocal;
     return $self->simpleContentExtension($tree->descend)
@@ -1036,7 +1049,7 @@ sub simpleContent($)
         if $name eq 'restriction';
 
      error __x"simpleContent either extension or restriction, not `{name}' at {where}"
-         , name => $name, where => $tree->path;
+         , name => $name, where => $tree->path, class => 'schema';
 }
 
 sub simpleContentExtension($)
@@ -1058,12 +1071,12 @@ sub simpleContentExtension($)
     my $basetype = $self->typeByName($tree, $typename);
     defined $basetype->{st}
         or error __x"base of simpleContent not simple at {where}"
-               , where => $where;
+             , where => $where, class => 'schema';
  
     $self->extendAttrs($basetype, $self->attributeList($tree));
     $tree->currentChild
         and error __x"elements left at tail at {where}"
-                , where => $tree->path;
+              , where => $tree->path, class => 'schema';
 
     $basetype;
 
@@ -1090,11 +1103,11 @@ sub simpleContentRestriction($$)
     else
     {   my $first    = $tree->currentLocal
             or error __x"no base in complex-restriction, so simpleType required at {where}"
-                   , where => $where;
+                 , where => $where, class => 'schema';
 
         $first eq 'simpleType'
             or error __x"simpleType expected, because there is no base attribute at {where}"
-                   , where => $where;
+                 , where => $where, class => 'schema';
 
         $type = $self->simpleType($tree->descend);
         $tree->nextChild;
@@ -1102,7 +1115,7 @@ sub simpleContentRestriction($$)
 
     my $st = $type->{st}
         or error __x"not a simpleType in simpleContent/restriction at {where}"
-               , where => $where;
+             , where => $where, class => 'schema';
 
     $type->{st} = $self->applySimpleFacets($tree, $st, 0);
 
@@ -1110,7 +1123,7 @@ sub simpleContentRestriction($$)
 
     $tree->currentChild
         and error __x"elements left at tail at {where}"
-                , where => $where;
+                , where => $where, class => 'schema';
 
     $type;
 }
@@ -1127,7 +1140,7 @@ sub complexContent($)
     
     $tree->nrChildren == 1
         or error __x"only one complexContent child expected at {where}"
-               , where => $tree->path;
+             , where => $tree->path, class => 'schema';
 
     my $name  = $tree->currentLocal;
  
@@ -1138,8 +1151,8 @@ sub complexContent($)
     return $self->complexBody($tree->descend)
         if $name eq 'restriction';
 
-    error __x"complexContent either extension or restriction, not '{name}' at {where}"
-        , name => $name, where => $tree->path;
+    error __x"complexContent expects either an extension or restriction, not `{name}' at {where}"
+        , name => $name, where => $tree->path, class => 'schema';
 }
 
 sub complexContentExtension($)
@@ -1154,7 +1167,7 @@ sub complexContentExtension($)
     {   my $typename = $self->rel2abs($where, $node, $base);
         my $typedef  = $self->namespaces->find(complexType => $typename)
             or error __x"unknown base type '{type}' at {where}"
-                   , type => $typename, where => $tree->path;
+                 , type => $typename, where => $tree->path, class => 'schema';
 
         $type = $self->complexType($tree->descend($typedef->{node}));
     }
@@ -1181,7 +1194,7 @@ sub rel2abs($$$)
     my $url = $node->lookupNamespaceURI($prefix);
 
     error __x"No namespace for prefix `{prefix}' in `{type}' at {where}"
-      , prefix => $prefix, type => $type, where => $where
+      , prefix => $prefix, type => $type, where => $where, class => 'schema'
         if length $prefix && !defined $url;
 
      pack_type $url, $local;
